@@ -22,7 +22,7 @@ import time
 
 import xgt
 import pyodbc
-from trovares_connector import ODBCConnector, SQLODBCDriver
+from trovares_connector.odbc import ODBCConnector, SQLODBCDriver
 
 class TestXgtODBCConnector(unittest.TestCase):
   # Print all diffs on failure.
@@ -305,3 +305,97 @@ class TestXgtODBCConnector(unittest.TestCase):
     self.conn.transfer_to_xgt(tables = [('test', ('Vertex1', 'Vertex2', 1, 2))], easy_edges = True, force = True)
     assert self.xgt.get_edge_frame('test').num_rows == 1
     print(self.xgt.get_edge_frame('test').get_data())
+
+  def test_transfer_query(self):
+    cursor = self.odbc_driver.cursor()
+    cursor.execute("CREATE TABLE test (TestBool BOOL, TestInt INT, TestBigInt BIGINT, TestFloat FLOAT(24), TestDouble FLOAT(53), "
+                   "TestFixedString char(5), TestString varchar(255), TestDecimal DECIMAL(10, 6), TestDate DATE, "
+                   "TestDatetime DATETIME, TestTimestamp TIMESTAMP NULL, TestTime TIME, TestYear YEAR)")
+    cursor.execute("INSERT INTO test VALUES (True, 32, 5000, 1.7, 1.98, 'vdxs', 'String', 1.78976, '1989-05-06', '1986-05-06 12:56:34', "
+                   "'1989-05-06 12:56:34', '12:56:34', 1999)")
+    cursor.execute("INSERT INTO test VALUES (NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)")
+    self.odbc_driver.commit()
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = 'my_test')
+    assert self.xgt.get_table_frame('my_test').num_rows == 2
+    print(self.xgt.get_table_frame('my_test').get_data())
+
+  def test_vertex_query(self):
+    cursor = self.odbc_driver.cursor()
+    cursor.execute("CREATE TABLE test (Value1 INT, Value2 INT, Value3 varchar(255))")
+    cursor.execute("INSERT INTO test VALUES (0, 0, 'hola')")
+    cursor.execute("INSERT INTO test VALUES (1, 0, 'adios')")
+    self.odbc_driver.commit()
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', (0,)))
+    assert self.xgt.get_vertex_frame('test').num_rows == 2
+    print(self.xgt.get_vertex_frame('test').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test1', (0,)))
+    assert self.xgt.get_vertex_frame('test1').num_rows == 2
+    print(self.xgt.get_vertex_frame('test1').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test2', ('Value1',)))
+    assert self.xgt.get_vertex_frame('test2').num_rows == 2
+    print(self.xgt.get_vertex_frame('test2').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', {'frame' : 'test3', 'key' : 0 } ))
+    assert self.xgt.get_vertex_frame('test3').num_rows == 2
+    print(self.xgt.get_vertex_frame('test3').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', {'frame' : 'test4', 'key' : 'Value1' } ))
+    assert self.xgt.get_vertex_frame('test4').num_rows == 2
+    print(self.xgt.get_vertex_frame('test4').get_data())
+
+  def test_edge_query(self):
+    cursor = self.odbc_driver.cursor()
+    cursor.execute("CREATE TABLE test (Value1 INT, Value2 INT, Value3 varchar(255))")
+    cursor.execute("INSERT INTO test VALUES (0, 0, 'hola')")
+    cursor.execute("INSERT INTO test VALUES (1, 0, 'adios')")
+    self.odbc_driver.commit()
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', ('Vertex', 'Vertex', 0, 1)), easy_edges=True)
+    assert self.xgt.get_edge_frame('test').num_rows == 2
+    print(self.xgt.get_edge_frame('test').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test1', ('Vertex', 'Vertex', 0, 1)))
+    assert self.xgt.get_edge_frame('test1').num_rows == 2
+    print(self.xgt.get_edge_frame('test1').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('Vertex1', (0,)))
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', 'test2', ('Vertex1', 'Vertex1', 'Value1', 'Value2')))
+    assert self.xgt.get_edge_frame('test2').num_rows == 2
+    print(self.xgt.get_edge_frame('test2').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', {'frame' : 'test3', 'source' : 'Vertex', 'target' : 'Vertex', 'source_key' : 0, 'target_key' : 1 }))
+    assert self.xgt.get_edge_frame('test3').num_rows == 2
+    print(self.xgt.get_edge_frame('test3').get_data())
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', {'frame' : 'test4', 'source' : 'Vertex', 'target' : 'Vertex', 'source_key' : 'Value1', 'target_key' : 'Value2' }))
+    assert self.xgt.get_edge_frame('test4').num_rows == 2
+    print(self.xgt.get_edge_frame('test4').get_data())
+
+  def test_append_query(self):
+    cursor = self.odbc_driver.cursor()
+    cursor.execute("CREATE TABLE test (Value1 INT, Value2 INT, Value3 varchar(255))")
+    cursor.execute("INSERT INTO test VALUES (0, 0, 'hola')")
+    self.odbc_driver.commit()
+
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', ('Vertex', 'Vertex', 0, 1)), easy_edges=True)
+    assert self.xgt.get_edge_frame('test').num_rows == 1
+    print(self.xgt.get_edge_frame('test').get_data())
+
+    cursor.execute("DROP TABLE IF EXISTS test")
+    cursor.execute("CREATE TABLE test (Value1 INT, Value2 INT, Value3 varchar(255))")
+    cursor.execute("INSERT INTO test VALUES (1, 0, 'adios')")
+    self.odbc_driver.commit()
+    self.conn.transfer_query_to_xgt("SELECT * FROM test", mapping = ('test', ('Vertex', 'Vertex', 0, 1)), append=True)
+    assert self.xgt.get_edge_frame('test').num_rows == 2
+    print(self.xgt.get_edge_frame('test').get_data())
+
+  def test_transfer_no_data_query(self):
+    c = self.conn
+    cursor = self.odbc_driver.cursor()
+    cursor.execute("CREATE TABLE Node (id INT)")
+    self.conn.transfer_query_to_xgt("SELECT * FROM Node", mapping = ('Node', (0,)))
+    assert self.xgt.get_vertex_frame('Node').num_rows == 0
